<?php

namespace Drupal\jsonapi_hypermedia\HypermediaProvider;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityPublishedInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\jsonapi\JsonApiResource\Link;
use Drupal\jsonapi\JsonApiResource\LinkCollection;
use Drupal\jsonapi\JsonApiResource\ResourceObject;
use Drupal\jsonapi_hypermedia\HypermediaProviderInterface;

/**
 * Class DefaultProvider.
 *
 * @internal
 */
final class DefaultProvider implements HypermediaProviderInterface {

  protected $entityTypeManager;

  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  public function hyperlink(LinkCollection $link_collection) {
    $context = $link_collection->getContext();
    if ($context instanceof ResourceObject) {
      $entity_type = $this->entityTypeManager->getDefinition($context->getResourceType()->getEntityTypeId());
      $entities = $this->entityTypeManager->getStorage($entity_type->id())->loadByProperties([$entity_type->getKey('uuid') => $context->getId()]);
      $entity = reset($entities);
      assert($entity instanceof EntityInterface);
      foreach ($link_collection as $key => $links) {
        if ($key === 'self') {
          $self_link = $links[0];
          assert($self_link instanceof Link);
          $update_access = $entity->access('update', NULL, TRUE);
          $delete_access = $entity->access('delete', NULL, TRUE);
          $cacheability = CacheableMetadata::createFromObject($self_link)
            ->addCacheableDependency($update_access)
            ->addCacheableDependency($delete_access);
          if ($entity instanceof EntityPublishedInterface) {
            $published_key = $entity_type->getKey('published');
            $moderate_access = $entity->access('update', NULL, TRUE)->andIf($entity->{$published_key}->access('edit', NULL, TRUE));
            $cacheability->addCacheableDependency($moderate_access);
            if ($moderate_access->isAllowed()) {
              $link_relations = ['https://jsonapi.org/profiles/drupal/hypermedia/#update'];
              $link_key = $entity->isPublished() ? 'unpublish' : 'publish';
              $attributes = [
                'linkParams' => [
                  'rel' => $link_relations,
                  'data' => [
                    'type' => $context->getTypeName(),
                    'id' => $context->getId(),
                    'attributes' => [
                      $published_key => (int) !$entity->isPublished(),
                    ]
                  ],
                ],
              ];
              $link_collection = $link_collection->withLink($link_key, new Link(CacheableMetadata::createFromObject($moderate_access), $self_link->getUri(), $link_relations, $attributes));
            }
          }
          $link_relations = $self_link->getLinkRelationTypes();
          if ($update_access->isAllowed()) {
            $link_relations[] = 'https://jsonapi.org/profiles/drupal/hypermedia/#update';
          }
          if ($delete_access->isAllowed()) {
            $link_relations[] = 'https://jsonapi.org/profiles/drupal/hypermedia/#delete';
          }
          // We have to call reset() on $link_relations to reset the array's
          // numerical index. Otherwise, it can output arrays with missing
          // indices and json_encode will serialize these as objects. This is a
          // band-aid. The real fix probably needs to be made in Link::merge(),
          // Link::__construct() or LinkCollection::withLink() because one of
          // these methods is likely where the array's pointer is getting out of
          // place.
          reset($link_relations);
          $link_relations = array_values($link_relations);
          $self_link = new Link($cacheability, $self_link->getUri(), $self_link->getLinkRelationTypes(), ['linkParams' => ['rel' => $link_relations]] + $self_link->getTargetAttributes());
          $link_collection = $link_collection->withLink('self', $self_link);
        }
      }
    }
    return $link_collection;
  }

}
