<?php

namespace Drupal\jsonapi\Normalizer\JsonApiHypermediaImpostor;

use Drupal\jsonapi\JsonApiResource\LinkCollection;
use Drupal\jsonapi\Normalizer\LinkCollectionNormalizer;
use Drupal\jsonapi_hypermedia\HypermediaProviderInterface;

class JsonApiHypermediaLinkCollectionNormalizer extends LinkCollectionNormalizer {

  /**
   * The JSON:API link collection normalizer.
   *
   * @var \Drupal\jsonapi\Normalizer\LinkCollectionNormalizer
   */
  protected $inner;

  /**
   * @var array
   */
  protected $providers;

  /**
   * @param \Drupal\jsonapi_hypermedia\HypermediaProviderInterface $provider
   * @param int $priority
   */
  public function addHypermediaProvider(HypermediaProviderInterface $provider, $priority = 0) {
    $this->providers[(int) $priority][] = $provider;
    krsort($this->providers, SORT_NUMERIC);
  }

  /**
   * The JSON:API link collection normalizer.
   *
   * @param \Drupal\jsonapi\Normalizer\LinkCollectionNormalizer $inner
   */
  public function setInnerNormalizer(LinkCollectionNormalizer $inner) {
    $this->inner = $inner;
  }

  /**
   * @inheritdoc
   */
  public function normalize($object, $format = NULL, array $context = []) {
    return $this->inner->normalize($this->getAlteredLinkCollection($object), $format, $context);
  }

  /**
   * Alters a JSON API link collection.
   *
   * @param \Drupal\jsonapi\JsonApiResource\LinkCollection $link_collection
   *   The collection to alter.
   *
   * @return \Drupal\jsonapi\JsonApiResource\LinkCollection
   *   The altered link collection.
   *
   * @throws
   */
  protected function getAlteredLinkCollection(LinkCollection $link_collection) {
    foreach ($this->providers as $priority => $providers) {
      foreach ($providers as $provider) {
        assert ($provider instanceof HypermediaProviderInterface);
        $link_collection = $provider->hyperlink($link_collection);
      }
    }
    return $link_collection;
  }

}
